# NOTE: Derived from /home/vfonov/src/minc-toolkit/mni-perllib/MNI/Spawn.pm.
# Changes made here will be lost when autosplit is run again.
# See AutoSplit.pm.
package MNI::Spawn;

#line 542 "/home/vfonov/src/minc-toolkit/mni-perllib/MNI/Spawn.pm (autosplit into /home/vfonov/src/build/minc-toolkit/mni-perllib/perl/auto/MNI/Spawn/check_program.al)"
# ------------------------------ MNI Header ----------------------------------
#@NAME       : check_program
#@INPUT      : $command - the full command
#              $program - the first word from a command, ie. the program
#                         to execute
#@OUTPUT     : 
#@RETURNS    : $fullpath    - full path (possibly relative) to the program;
#                             if the input $program was a path, then this
#                             will be identical to it
#              $pre_defargs - [array ref] list of options associated with
#                             this program
#              $post_defargs- [array ref] list of options associated with
#                             this program that are meant to come at the
#                             *end* of the command line
#@DESCRIPTION: Makes sure that a program we wish to execute actually exists.
#              How this is done depends on how the program name is specified;
#              if it is just a filename (no slashes), then we first look
#              in the 'programs' hash to see if it was registered via
#              register_programs; if not, we (optionally) complain and try to
#              find out ourselves via an explicit search.  If this fails,
#              we fail via &check_status (meaning we might just
#              return 0, or completely bomb out, or whatever the caller
#              asked for).
#
#              If, however, the program is supplied as a path of some sort
#              (ie. there're slashes in it), then we simply check that the
#              specified file exists and is executable.  If not, we again
#              bomb via &check_status.
#@METHOD     : 
#@GLOBALS    : %Programs, %PreOptions, %PostOptions
#@CALLERS    : complete_command
#@CALLS      : MNI::FileUtilities::find_program
#              MNI::PathUtilities::split_path
#              check_status
#@CREATED    : 1996/11/19, GPW (from prototype code in &Execute)
#@MODIFIED   : 1997/07/08, GPW (copied from JobControl.pm)
#-----------------------------------------------------------------------------
sub check_program
{
   my ($self, $command, $program) = @_;
   my ($fullpath, @pre_defargs, @post_defargs);

   # If $program was supplied as a bare filename, then we poke around a bit
   # to find it.  First try the `programs' hash; if it's not found there,
   # then we issue a warning (if the `strict' flag is set) and try to find
   # it ourselves with &find_program.  If this also fails, then we call
   # &check_status with an artificial failure status (the same status that
   # Perl sets if system() fails because the program doesn't exist) to
   # provoke the appropriate failure action.  It's ok to do this without
   # printing "$program not found" because find_program does this for us.

   if ($program !~ m|/|)      # just a name, no directories at all
   {
      $fullpath = $self->{programs}{$program};
      if (! defined $fullpath) 
      {
         croak ("spawn: error: program \"$program\" not registered")
            if ($self->{strict} == 2);
         carp ("spawn: warning: program \"$program\" not registered")
            if ($self->{strict} == 1);

         if ($self->{search})
         {
            my @warnings;
            @warnings = catch_warnings 
               (sub {
                  $fullpath = find_program ($program, $self->{search_path}) 
                });

            if (! $fullpath)
            {
               map { warn "spawn: warning: \l$_" } @warnings;
               $self->check_status (255 << 8, $program, $command, 
                                    UNTOUCHED, UNTOUCHED);
               return ();
            }
         }
         else                           # program not registered, but caller
         {                              # disallowed searching -- so mere
            $fullpath = $program;       # program name will have to do
         }
      }
   }
   else                                 # caller supplied a path (possibly
   {                                    # relative, but that doesn't matter)
      unless (-e $program && -x $program)
      {
         warn "spawn: warning: " . 
              "program \"$program\" doesn't exist or isn't executable\n";
         return $self->check_status (255 << 8, $program, $command,
                                    UNTOUCHED, UNTOUCHED);
      }

      $fullpath = $program;
      $program = (split_path ($fullpath, 'none'))[1];
   }

   # Now, $program is base filename, and $fullpath is the full path to it
   # (possibly relative, if that's what we were supplied with; or it might
   # just be the bare program name, if we should have searched but the
   # caller disallowed that).  Look up program options in the two options
   # hashes for use by the caller.

   if ($self->{add_defaults})
   {
      @pre_defargs = (exists $self->{defargs}{pre}{$program})
         ? @{$self->{defargs}{pre}{$program}}
         : ();
      @post_defargs = (exists $self->{defargs}{post}{$program})
         ? @{$self->{defargs}{post}{$program}}
         : ();
   }

   ($fullpath, \@pre_defargs, \@post_defargs);

}  # &check_program

# end of MNI::Spawn::check_program
1;
