# NOTE: Derived from /home/vfonov/src/minc-toolkit/mni-perllib/MNI/Spawn.pm.
# Changes made here will be lost when autosplit is run again.
# See AutoSplit.pm.
package MNI::Spawn;

#line 1096 "/home/vfonov/src/minc-toolkit/mni-perllib/MNI/Spawn.pm (autosplit into /home/vfonov/src/build/minc-toolkit/mni-perllib/perl/auto/MNI/Spawn/check_status.al)"
# ------------------------------ MNI Header ----------------------------------
#@NAME       : &check_status
#@INPUT      : $status
#              $program
#              $command
#              $stdout
#              $stderr
#              $output
#              $error
#@OUTPUT     : 
#@RETURNS    : 
#@DESCRIPTION: Checks the termination status of a program.  If it is zero
#              (the program exited normally, indicating "success" through a
#              zero exit status), we do nothing and return true.  (Note
#              that `spawn' then returns the termination status, which in
#              this case will be zero, to its caller.)
# 
#              If the termination status was non-zero, carries out the
#              instructions implicit in the `err_action' option.
#              Currently the following values are supported for
#              `err_action':
#                 `fatal'   - crash the whole program immediately
#                 `notify'  - send email to the address specified in the
#                             `notify' option, and *then* crash the whole
#                             program
#                 `warn'    - print a warning and return false
#                 `ignore'  - say nothing and return false
#
#              Alternately, `err_action' can be an arbitrary chunk of Perl
#              code, which is evaluated in the calling package; if there
#              are errors in this code, we again crash by calling croak.
#              Beware of making this code a single string of lowercase
#              letters, as I reserve the right to add more options like
#              `fatal' or `notify' that might cause conflicts.
#
#              Finally, if `err_action' is not supplied (eg. undefined or
#              the empty string), we act as though it were really `warn'
#              (for backwards compatibility).  Again, note that the return
#              value is turned around by `spawn' -- it will return the
#              termination status to its caller, which this time is
#              non-zero.
#@METHOD     : 
#@GLOBALS    : 
#@CALLERS    : 
#@CALLS      : 
#@CREATED    : 1996/01/17, GPW (from code in &Spawn)
#@MODIFIED   : 1997/07/08, GPW (from JobControl.pm)
#-----------------------------------------------------------------------------
sub check_status
{
   my ($self, $status, $program, $command,
       $stdout_mode, $stderr_mode, $output, $error) = @_;

   # Note to myself: $status is now the full 16-bit value as returned by
   # wait(2) (the termination status, as opposed to the exit status which
   # is passed to exit()).  I should do some bit-twiddling (preferably with
   # the help of "sys/wait.ph", but it's not cooperating right now) to
   # figure out what kind of crash, exit status, signal number, etc.

   if ($status)
   {
      my $ea = $self->{err_action};
      $ea = 'warn' unless $ea;          # backwards compatibility (empty string
                                        # or undefined same as 'warn')
      $! = $? >> 8;                     # so exit status will propagate

      if ($ea eq 'fatal')
      {
         die "$ProgramName: crashed while running $program " .
             "(termination status=$status)\n";
      }
      elsif ($ea eq 'notify')
      {
         $self->obituary ($status, $program, $command,
                          $stdout_mode, $stderr_mode, $output, $error);
      }
      elsif ($ea eq 'ignore')
      {
         return 0;
      }
      elsif ($ea eq 'warn')
      {
         warn "$ProgramName: warning: " .
              "$program crashed (termination status=$status)\n";
         return 0;
      }
      elsif ($ea =~ /^[a-z_]+$/)        # looks like some other keyword
      {
         croak "spawn: unknown err_action keyword \"$ea\"";
      }
      elsif ($ea)                       # some chunk of code to be eval'd
      {
         carp "spawn: you should be using `notify' rather than " .
               "hard-coding a call to \&Obituary in `err_action'" 
            if $ea =~ /obituary/i;
         carp "spawn: you should be using `fatal' rather than " .
              "hard-coding a `die' in `err_action'" 
            if $ea =~ /die/;

         my $up_pkg = find_calling_package;

	 eval "package $up_pkg; $ea";
	 croak "spawn: error in err_action code: $@" if $@;
         return 0;
      }
   }
   else
   {
      return 1;
   }

}  # check_status

# end of MNI::Spawn::check_status
1;
