# NOTE: Derived from /home/vfonov/src/minc-toolkit/mni-perllib/MNI/Spawn.pm.
# Changes made here will be lost when autosplit is run again.
# See AutoSplit.pm.
package MNI::Spawn;

#line 660 "/home/vfonov/src/minc-toolkit/mni-perllib/MNI/Spawn.pm (autosplit into /home/vfonov/src/build/minc-toolkit/mni-perllib/perl/auto/MNI/Spawn/complete_command.al)"
# ------------------------------ MNI Header ----------------------------------
#@NAME       : complete_command
#@INPUT      : $command  - [list ref or string]
#              $verbose  - true to print out full command
#@OUTPUT     : 
#@RETURNS    : $command  - [list ref or string -- whichever the input is]
#                          input $command with full path to program and
#                          any standard options included
#              $program  - the program name extracted from $command
#@DESCRIPTION: Extracts the program name from a command (list or string),
#              calls &check_program to get the full path and options
#              for the program, and constructs a new command (list or
#              string) with these goodies included.
#@METHOD     : 
#@GLOBALS    : 
#@CALLERS    : spawn
#@CALLS      : check_program
#              usestamp, timestamp, shellquote (from MNI::MiscUtilities)
#@CREATED    : 1996/12/10, GPW (from &Execute)
#@MODIFIED   : 1997/07/08, GPW (copied from JobControl.pm)
#-----------------------------------------------------------------------------
sub complete_command
{
   my ($self, $command, $verbose) = @_;
   my ($complete, $program, $pre_defargs, $post_defargs);

   if (ref $command)
   {
      croak ("spawn: \$command must be either a list ref or a simple string")
         unless (ref $command eq 'ARRAY');

      # Make our own copy of the command list to scribble on (and eventually
      # return to `spawn')
         
      my @command = @$command;

      # Check for "escaped" command (starts with a backslash) -- if found,
      # remove the backslash, warn that this is obsolete, and clear $complete.
      # XXX the "escape" hack should eventually be removed!
      
      if ($command[0] =~ s|^\\||)
      {
         carp "spawn: warning: escaping commands with backslash is deprecated".
              " -- you should instead set the `complete' option to false";
         $complete = 0;
      }
      else                              # otherwise the `complete' option
      {                                 # is used
         $complete = $self->{complete};
      }

      # Now we actually pay attention to $complete (either from the
      # escapedness of the command or, usually, from the `complete'
      # option) -- only do the fancy stuff (search, add options) if
      # it's true

      if ($complete)
      {
         $program = shift @command;
         my ($fullpath, $pre_defargs, $post_defargs) =
            $self->check_program ($command, $program);
         return () unless defined $fullpath;
         
         # Build a new command list using the full path and default options;
         # use this to print and execute
         
         unshift (@command, $fullpath, @$pre_defargs);
         push (@command, @$post_defargs);
      }
      else
      {
         $program = $command[0];
      }

      # Finally, print the command if the Verbose flag is true

      if ($verbose)
      {
         my $lh = $self->{loghandle};
         printf $lh ("[%s] [%s] [%s] %s\n",
                     $ProgramName, userstamp(), timestamp(),
                     shellquote (@command));
      }

      # And return the augmented command list

      return (\@command, $program);
   }
   else                                 # $command is just a string
   {
      if ($command =~ s|^\\||)
      {
         carp "spawn: warning: escaping commands with backslash is deprecated".
              " -- you should instead set the `complete' option to false";
         $complete = 0;
      }
      else                              # otherwise the `complete' option
      {                                 # is used
         $complete = $self->{complete};
      }

      if ($complete)
      {
         # Get the first word (i.e. the program to run) from the command
         # string; we do this via a regexp rather than split so we can
         # remove the program name and put it back in later, with default
         # options added

         # XXX uh... what if $command is "ls>foo" ??? -- this regexp won't
         # do the right thing!

         my $args;
         ($args = $command) =~ s/^(\S+)\s*//;
         $program = $1;
         my ($fullpath, $pre_defargs, $post_defargs) = 
            $self->check_program ($command, $program);
         return () unless defined $fullpath;

         # Build a new command string using the full path and default options;
         # print the command and execute it

         $command = join (" ", $fullpath,
                          @$pre_defargs, $args || (), @$post_defargs);
      }
      else
      {
         ($program) = $command =~ /^(\S+)\s*/;
      }
         

      if ($verbose)
      {
         my $lh = $self->{loghandle};
         printf $lh ("[%s] [%s] [%s] %s\n",
                     $ProgramName, userstamp(), timestamp(), $command);
      }

      return ($command, $program);
   }

}  # complete_command

# end of MNI::Spawn::complete_command
1;
