# NOTE: Derived from /home/vfonov/src/minc-toolkit/mni-perllib/MNI/Spawn.pm.
# Changes made here will be lost when autosplit is run again.
# See AutoSplit.pm.
package MNI::Spawn;

#line 852 "/home/vfonov/src/minc-toolkit/mni-perllib/MNI/Spawn.pm (autosplit into /home/vfonov/src/build/minc-toolkit/mni-perllib/perl/auto/MNI/Spawn/exec_command.al)"
# ------------------------------ MNI Header ----------------------------------
#@NAME       : &exec_command
#@INPUT      : $command - the whole command, either as string or list ref
#              $program - the program name
#              $stdout  - where to redirect stdout to
#              $stderr  - where to redirect stderr to (or "-" to emulate
#                         capturing it by redirecting to temp file)
#@OUTPUT     : 
#@RETURNS    : does not return -- either execs successfully or dies
#@DESCRIPTION: Run by the child process after we fork -- this takes care
#              of redirecting stdout (if appropriate) and stderr (likewise),
#              exec's the command, and bombs if the exec fails.
#@CALLERS    : spawn_redirect, spawn_capture
#@CALLS      : 
#@CREATED    : 1996/12/10, GPW
#@MODIFIED   : 1997/07/08, GPW (from JobControl.pm)
#-----------------------------------------------------------------------------
sub exec_command
{
   my ($command, $program, $stdout_mode, $stdout, $stderr_mode, $stderr) = @_;

   # Redirect stdout and/or stderr if the caller specified file
   # destination(s) (also possibly "capture" stderr via temp file)

   if ($stdout_mode == REDIRECT)
   {
      # Note to myself: it's possible to spawn a child with stdout 
      # redirected to the same file as the parent is currently being
      # logged to.  For instance, if the user runs "parent > log", and
      # then spawns a child with stdout => 'log', this will happen.
      # This is a bad thing -- the child's output gets all munged up.
      # This happens even if the user ran "parent >> log".  It can 
      # be detected, though -- just stat STDOUT and the caller-supplied
      # log file; if the device and inode numbers are the same, we should
      # print a warning before forking or redirecting.  HmmmMMMmmmm...

      # Incidentally, if the parent forks a child with stdout => 'log',
      # and then later on redirects its own stdout to the same place,
      # all works out fine.  (Although I have only tried with both
      # processes opening 'log' in append mode.)  HmmmMMMmm again!

      open (STDOUT, $stdout)
         || croak ("spawn: unable to redirect stdout to \"$stdout\": $!");
   }

   my $stderr_dest;
   if ($stderr_mode == REDIRECT)    { $stderr_dest = $stderr }
   elsif ($stderr_mode == CAPTURE)  { $stderr_dest = ">/tmp/error$$.log" }
   elsif ($stderr_mode == MERGE)    { $stderr_dest = ">&STDOUT" }

   if ($stderr_dest)
   {
      open (STDERR, $stderr_dest)
         || croak ("spawn: unable to redirect stderr to \"$stderr_dest\": $!");
   }

   # Exec that sucker; we turn warnings off because Perl generates a
   # warning on failed exec, which would be redundant (due to 'die' below)

   {
      local $^W = 0;
      (ref $command)
         ? exec @$command
         : exec $command;
   }

   # If we get here, the exec failed -- this should not normally happen
   # because of the care we take to find the program in &check_program, but
   # could if the user has disabled searching

   print STDERR "spawn: exec of $program failed: $!\n";
   exit 255;

}  # exec_command

# end of MNI::Spawn::exec_command
1;
