# NOTE: Derived from /home/vfonov/src/minc-toolkit/mni-perllib/MNI/Spawn.pm.
# Changes made here will be lost when autosplit is run again.
# See AutoSplit.pm.
package MNI::Spawn;

#line 311 "/home/vfonov/src/minc-toolkit/mni-perllib/MNI/Spawn.pm (autosplit into /home/vfonov/src/build/minc-toolkit/mni-perllib/perl/auto/MNI/Spawn/spawn.al)"
# ------------------------------ MNI Header ----------------------------------
#@NAME       : spawn
#@INPUT      : 
#@OUTPUT     : 
#@RETURNS    : 
#@DESCRIPTION: 
#@CREATED    : 1997/07/07, GPW, from Spawn in JobControl.pm
#@MODIFIED   : 
#-----------------------------------------------------------------------------
sub spawn
{
   my ($self, $command, @options) = @_;
   my ($program);

   # If caller supplied any options, make a copy of the spawning vat
   # and override those options in the copy only

   if (@options)
   {
      $self = $self->copy;
      $self->set_options (@options);
   }


   # Inherit `verbose' and `execute' options from variables in calling
   # package if they aren't already defined

   $self->set_undefined_option ('verbose', 'Verbose');
   $self->set_undefined_option ('execute', 'Execute');


   # Complete the command (ie. program name to full path, insert
   # options fore and aft)
   # 
   # if in batch mode, then let batch print the commands it executes

   ($command, $program) = $self->complete_command
      ($command, $self->{verbose} && !$self->{batch});
   return 1 unless defined $command;

   # Figure out just what the user wants us to do with stdout and stderr

   my ($stdout, $stderr) = @$self{'stdout','stderr'};
   my ($stdout_mode, $stderr_mode);
   ($stdout, $stdout_mode) = output_mode ('stdout', $stdout);
   croak "spawn: you can't merge stdout with itself!"
      if $stdout_mode == MERGE;
   ($stderr, $stderr_mode) = output_mode ('stderr', $stderr);


   # Determine if we should merge stderr with stdout: first, the caller
   # must not have explicitly specified what to do with stderr already; and
   # second, we must be redirecting stdout.  Note that this is the only
   # place where we distinguish between the empty string and undef for
   # stderr; the empty string means "definitely leave it untouched", and
   # undef means "maybe leave it untouched, maybe merge it".  (output_mode
   # treates them as equivalent -- both result in $stderr_mode being set to
   # UNTOUCHED.)

   if (!defined $stderr && $stdout_mode == REDIRECT) 
   {
      $stderr_mode = MERGE;
   }

   # Figure out how to open files ('>' to overwrite or '>>' to append), and
   # prepend that to $stdout and $stderr if they don't already have such a
   # code in them.

   my $open_prefix = $self->{clobber} ? '>' : '>>';
   $stdout = $open_prefix . $stdout 
      if ($stdout_mode == REDIRECT && $stdout !~ /^>/);
   $stderr = $open_prefix . $stderr 
      if ($stderr_mode == REDIRECT && $stderr !~ /^>/);


   # Warn if the user is calling us in array context

   carp "spawn: called in an array context (no longer useful)"
      if wantarray;


   # Return now unless the `execute' option is true.  (Note that spawn's
   # return value is bass-ackwards: 0 implies success, just like a
   # program's exit status [this is not coincidental!].)

   return 0 unless $self->{execute};


   # Figure out if the child should announce itself -- here we conspire
   # with self_announce in MNI::Startup via an environment variable,
   # inspirationally named `self_announce'.  It works as follows: here,
   # we set `self_announce' to true or false, depending on whether the
   # child should or should not announce its arguments to the world.
   # (Basically, it should always do so, unless its output is being
   # captured, or its output is not being redirected and we -- the
   # parent -- have already printed the child command line.)  Then, in
   # self_announce, the value of this variable is checked.  If it
   # doesn't exist at all, then the child will assume it run by some
   # non-Spawn-using program, and thus will announce itself.  Otherwise,
   # it takes the advice of the environment variable and either
   # announces itself or not.  (One subtlety: self_announce will not
   # announce if its stdout is a tty.)

#    false if $stdout_mode == CAPTURE;
#    false if verbose and $stdout_mode == UNTOUCHED;
#    [false if verbose and $stdout_mode == REDIRECT and stdout == loghandle]
#    true otherwise.

   $ENV{'suppress_announce'} = 
      ($stdout_mode == CAPTURE) ||
      ($self->{'verbose'} && $stdout_mode == UNTOUCHED);


   # Now, we finally get to run the command.  This is done via either
   # spawn_capture or spawn_redirect, depending on what we're doing with
   # stdout.  (Doing nothing is handled by spawn_redirect.)

   if ($stdout_mode == CAPTURE)         # capturing to a variable?
   {
      $self->spawn_capture ($command, $program,
                            $stdout_mode, $stdout, $stderr_mode, $stderr);
   }
   else
   {
      $self->spawn_redirect ($command, $program,
                             $stdout_mode, $stdout, $stderr_mode, $stderr);
   }
}

# end of MNI::Spawn::spawn
1;
