# NOTE: Derived from /home/vfonov/src/minc-toolkit/mni-perllib/MNI/Spawn.pm.
# Changes made here will be lost when autosplit is run again.
# See AutoSplit.pm.
package MNI::Spawn;

#line 1210 "/home/vfonov/src/minc-toolkit/mni-perllib/MNI/Spawn.pm (autosplit into /home/vfonov/src/build/minc-toolkit/mni-perllib/perl/auto/MNI/Spawn/spawn_capture.al)"
# ------------------------------ MNI Header ----------------------------------
#@NAME       : &spawn_capture
#@INPUT      : $command - [list ref or string] the full command
#              $program - the program name, as extracted by &complete_command
#              $stderr  - what to do with stderr.  Possible values:
#                  scalar ref : capture to referenced variable
#                  a string   : redirect to the named file
#@OUTPUT     : 
#@RETURNS    : $status - termination status of $command, ie. non-zero means
#                        failure or abnormal termination
#@DESCRIPTION: Run a command, capturing stdout to a variable.  stderr might
#              also be captured, but then again it might be redirected
#              (including possibly merged with stdout) or left intact.
#@METHOD     : forks and execs desired command with a pipe between parent
#              and child; captures stderr via a temporary file if it has
#              to
#@GLOBALS    : 
#@CALLERS    : spawn
#@CALLS      : flush_stdout, exec_command, gather_error, check_status
#@CREATED    : 1996/12/10, GPW (from &Spawn)
#@MODIFIED   : 1997/07/08, GPW (from JobControl.pm)
#-----------------------------------------------------------------------------
sub spawn_capture
{
   my ($self, $command, $program,
       $stdout_mode, $stdout, $stderr_mode, $stderr) = @_;
   my ($pid, $status);

   # Run the command through a pipe.  In the child, stdout automagically
   # goes to the pipe; in the parent, we will slurp the child's output from
   # the pipe.  stderr is dealt with according to $stderr_mode -- either
   # it's captured (via a temporary file), or it's redirected to some other
   # file, or it's merged with stdout (which is actually handled as a
   # redirect to "&STDOUT" -- this is done by exec_command), or it's
   # left untouched.

   flush_stdout;
   $pid = open (PIPE, "-|");
   croak "spawn: failed to start child process: $!" unless defined $pid;

   if ($pid == 0)                       # if in the child, then exec --
   {                                    # this doesn't return!
      exec_command ($command, $program,
                    $stdout_mode, $stdout, $stderr_mode, $stderr);
   }

   # $pid is not 0, so we're in the parent.  We read in the child's stdout
   # (and stderr too if it was captured).

   capture_stream (\*PIPE, $stdout);
   close (PIPE);
   $status = $?;                        # get child's termination status

   if ($stderr_mode == CAPTURE)         # did we "capture" stderr?
   {                                    # then read it in from temp file
      gather_error ($pid, $stderr);
   }

   $self->check_status ($status, $program, $command,
                        $stdout_mode, $stderr_mode,
                        $stdout, 
                        ($stderr_mode == CAPTURE) ? $stderr : undef);

   return $status;
}

# end of MNI::Spawn::spawn_capture
1;
