# NOTE: Derived from /home/vfonov/src/minc-toolkit/mni-perllib/MNI/Spawn.pm.
# Changes made here will be lost when autosplit is run again.
# See AutoSplit.pm.
package MNI::Spawn;

#line 1277 "/home/vfonov/src/minc-toolkit/mni-perllib/MNI/Spawn.pm (autosplit into /home/vfonov/src/build/minc-toolkit/mni-perllib/perl/auto/MNI/Spawn/spawn_redirect.al)"
# ------------------------------ MNI Header ----------------------------------
#@NAME       : &spawn_redirect
#@INPUT      : $command - [list ref or string] the full command
#              $program - the program name, as extracted by &complete_command
#              $stdout  - what to do with stdout
#              $stderr  - what to do with stderr
#@OUTPUT     : 
#@RETURNS    : $status  - termination status of $command, ie. non-zero means
#                         failure or abnormal termination
#@DESCRIPTION: Run a command, redirecting stdout to a file or leaving it
#              intact.  stderr might likewise be redirected to file, merged
#              with stdout, captured to a variable (via a temporary file),
#              or left untouched.
#@METHOD     : 
#@GLOBALS    : 
#@CALLERS    : spawn
#@CALLS      : flush_stdout, exec_command, gather_error, check_status
#@CREATED    : 1996/12/10, GPW (from &Spawn)
#@MODIFIED   : 1997/07/08, GPW (from JobControl.pm)
#-----------------------------------------------------------------------------
sub spawn_redirect
{
   my ($self, $command, $program,
       $stdout_mode, $stdout, $stderr_mode, $stderr) = @_;
   my ($pid, $status);

   flush_stdout;
   $pid = fork;
   croak "spawn: failed to start child process: $!" unless defined $pid;

   if ($pid == 0)                       # if in the child, then exec --
   {                                    # this doesn't return!
      exec_command ($command, $program,
                    $stdout_mode, $stdout, $stderr_mode, $stderr);
   }


   # $pid is not 0, so we're in the parent; block and wait for child.

   if (waitpid ($pid, 0) == -1)
   {
      confess "spawn: no child processes even though I just forked!";
   }

   $status = $?;

   if ($stderr_mode == CAPTURE)
   {
      gather_error ($pid, $stderr);
   }

   $self->check_status ($status, $program, $command,
                        $stdout_mode, $stderr_mode,
                        undef,
                        ($stderr_mode == CAPTURE) ? $stderr : undef);

   return $status;
}

# end of MNI::Spawn::spawn_redirect
1;
