# NOTE: Derived from /home/vfonov/src/minc-toolkit/mni-perllib/MNI/Startup.pm.
# Changes made here will be lost when autosplit is run again.
# See AutoSplit.pm.
package MNI::Startup;

#line 803 "/home/vfonov/src/minc-toolkit/mni-perllib/MNI/Startup.pm (autosplit into /home/vfonov/src/build/minc-toolkit/mni-perllib/perl/auto/MNI/Startup/backgroundify.al)"
# ------------------------------ MNI Header ----------------------------------
#@NAME       : backgroundify
#@INPUT      : $log     - either a filename or filehandle
#              $program - [optional] name of program to announce; default $0
#              $args    - [list ref; optional] list of arguments to announce
#                         with $program; defaults to @ARGV
#@OUTPUT     : 
#@RETURNS    : 
#@DESCRIPTION: Redirects STDOUT and STDERR to $log, forks, and (in the 
#              parent) exits.  Returns 1 to newly forked child process 
#              on success; dies on any error.  (No errors are possible
#              after the fork, so only the parent process will die.)
#
#              This is *not* sufficient for writing a daemon.
#@METHOD     : 
#@GLOBALS    : $0, @ARGV, STDOUT, STDERR
#              $Verbose, $Clobber
#@CALLS      : self_announce
#@CREATED    : 1997/07/28, GPW (loosely based on code from old JobControl.pm)
#@MODIFIED   : 
#-----------------------------------------------------------------------------
sub backgroundify
{
   my ($log, $program, $args) = @_;
   my ($stdout, $log_existed);

   # XXX to emulate what happens when a shell puts something in 
   # the BG, should we be be calling setpgrp or something???

   select STDERR; $| = 1;
   select STDOUT; $| = 1;

   # First, figure out what the nature of $log is.  We assume that if it's
   # a reference, it must be a filehandle in some form.  

   if (ref $log)                        # probably a filehandle or something
   {
      croak "backgroundify: if supplied, \$log must be an open filehandle " .
            "or a filename"
         unless defined fileno ($log);
      carp "backgroundify: \$log should not be connected to a TTY"
         if -t fileno ($log);

      $stdout = '>&=' . fileno ($log);
      print "$ProgramName: redirecting output " .
            "and detaching to background\n"
         if $Verbose;
   }
   elsif ($log)                         # assume it's a filename
   {
      if ($log =~ /^>/)                 # user already supplied clobber 
         { $stdout = $log }             # or append notation
      else                              # else, we have to figure it out
         { $stdout = ($Clobber ? '>' : '>>') . $log }
         
      $log_existed = -e $log;
      print "$ProgramName: redirecting output to $log " .
            "and detaching to background\n"
         if $Verbose;
   }
   else
   {
      croak "backgroundify: \$log must be a filehandle (glob ref) or filename";
   }

   # First save the current destination of stdout and stderr; they will be
   # restored in the parent, in case the `exit' we do there causes any
   # output.  (This can be important, because the user might have END
   # blocks in his program that will -- if he's not careful -- be executed
   # by both the parent and the child.  Restoring stdout and stderr before
   # we `exit' might help this sort of mistake get caught.)

   local (*SAVE_STDOUT, *SAVE_STDERR);
   open (SAVE_STDOUT, ">&STDOUT") || die "couldn't save STDOUT: $!\n";
   open (SAVE_STDERR, ">&STDERR") || die "couldn't save STDERR: $!\n";

   # Now redirect stdout and stderr.  We do this before forking (thus
   # necessitating the save-and-restore code) because redirection is more
   # likely to cause errors than forking, and we want any such error
   # messages to appear on the original stderr (for immediate visibility)
   # rather than in the log file if at all possible.

   unless (open (STDOUT, $stdout))
   {
      die "$ProgramName: detachment to background failed: couldn't redirect stdout to \"$stdout\" ($!)\n";
   }
   unless (open (STDERR, ">&STDOUT"))
   {
      die "$ProgramName: detachment to background failed: couldn't redirect stderr into stdout ($!)\n";
   }

   my $pid = fork;
   die "$ProgramName: detachment to background failed: couldn't fork: $!\n"
      unless defined $pid;

   if ($pid)                            # in the parent (old process)?
   {
      @options{'cputimes','cleanup'} = (); # disable normal shutdown sequence
      open (STDOUT, ">&SAVE_STDOUT") || die "couldn't restore STDOUT: $!\n";
      open (STDERR, ">&SAVE_STDERR") || die "couldn't restore STDERR: $!\n";
      exit;                             # and exit
   }

   # Now, we're in the child (new process) -- reset the "time used"
   # counters, start a new process group (to emulate what the shells do
   # when forking a child), print "self announcement" to (redirected)
   # stdout, and carry on as usual

   @start_times = times
      if ($options{'cputimes'});
   setpgrp;
   self_announce (\*STDOUT, $program, $args, 1);

   return 1;                            # return OK in new process

}  # backgroundify

1;
1;
# end of MNI::Startup::backgroundify
