# NOTE: Derived from /home/vfonov/src/minc-toolkit/mni-perllib/MNI/Startup.pm.
# Changes made here will be lost when autosplit is run again.
# See AutoSplit.pm.
package MNI::Startup;

#line 421 "/home/vfonov/src/minc-toolkit/mni-perllib/MNI/Startup.pm (autosplit into /home/vfonov/src/build/minc-toolkit/mni-perllib/perl/auto/MNI/Startup/startup.al)"
# &startup is where we actually "do some work", i.e. set all the global
# variables that we exported up in `import'.  It's a separate subroutine
# (called by import) because it has to be done *after* import is called,
# and import isn't called until the module has been require'd
# (i.e. compiled and run).

sub startup
{
   # We set $ProgramDir and $ProgramName regardless of the options in the
   # import list because $ProgramName is needed for the temp dir name and
   # various handy messages to the user.  Likewise for $StartDir and
   # $StartDirName -- $StartDir is needed by self_announce and to cleanup
   # safely (if $TmpDir is a relative path), so we always set it too.  The
   # `progname' and `startdir' options only control whether these variables
   # are exported into the user's namespace, which is controlled by `import'
   # above.

   ($ProgramDir,$ProgramName) = $0 =~ m|^(.*/)?([^/]*)$|;
   $ProgramDir = '' unless defined $ProgramDir;

   # We need to find the starting directory if the 'startdir' option is
   # true, OR if we're going to need it later to make $TmpDir absolute.
   # The latter is true when the TMPDIR environment variable is defined
   # but not an absolute path.

   if ($options{startdir} ||
       ($ENV{'TMPDIR'} && substr ($ENV{'TMPDIR'}, 0, 1) ne '/'))
   {
      # This little trickery lets us get away with not 'use'ing Cwd --
      # we make Carp export its symbols into Cwd at compile-time, but
      # don't load Cwd.pm unless it's definitely needed.  This can 
      # shave a few tenths of a second off the overhead of using
      # MNI::Startup.

      BEGIN { package Cwd; import Carp; }
      require Cwd;

      # Note that if the cwd on startup is '/', $StartDirName will be
      # undefined.  This makes sense to me, as there is simply no
      # "trailing name" component in '/' -- we can't just pretend it's
      # there but empty (like with a missing directory component).

      $StartDir = Cwd::getcwd ();
      $StartDir .= '/' unless substr ($StartDir, -1, 1) eq '/';
      ($StartDirName) = $StartDir =~ m|/([^/]+)/$|;
   }

   if ($options{optvars})
   {
      $Verbose = 1;
      $Execute = 1;
      $Clobber = 0;
      $Debug = 0;

      my $basetmp;

      if (defined($ENV{'TMPDIR'})) {
	$basetmp = $ENV{'TMPDIR'};
      } elsif ( -d '/usr/tmp' ) {
	$basetmp = '/usr/tmp';
      } elsif ( -d '/var/tmp' ) {
	$basetmp = '/var/tmp';
      } elsif ( -d '/tmp' ) {
	$basetmp = '/tmp';
      }

      $basetmp = $StartDir . $basetmp unless substr ($basetmp, 0, 1) eq '/';
      $basetmp .= '/' unless substr ($basetmp, -1, 1) eq '/';
      $TmpDir = ($basetmp . "${ProgramName}_$$/");
      croak "$ProgramName: temporary directory $TmpDir already exists"
         if -e $TmpDir;
      $orig_tmpdir = $TmpDir;
      $KeepTmp = 0;
   }

   if ($options{opttable})
   {
      @DefaultArgs =
         (['Basic behaviour options', 'section'],
          ['-verbose|-quiet', 'boolean', 0, \$Verbose, 
           'print status information and command lines of subprograms ' .
           '[default; opposite is -quiet]' ],
          ['-execute', 'boolean', 0, \$Execute, 
           'actually execute planned commands [default]'],
          ['-clobber', 'boolean', 0, \$Clobber,
           'blithely overwrite files (and make subprograms do as well) ' .
           '[default: -noclobber]'],
          ['-debug', 'boolean', 0, \$Debug,
           'spew lots of debugging info (and make subprograms do so as well)' .
           ' [default: -nodebug]'],
          ['-tmpdir', 'string', 1, \$TmpDir,
           'set the temporary working directory'],
          ['-keeptmp|-cleanup', 'boolean', 0, \$KeepTmp,
           'don\'t delete temporary files when finished ' .
           '[default: -nokeeptmp]']);
   }

   if ($options{cputimes})
   {
      @start_times = times;
   }

   if ($options{sig})
   {
      my ($sig, @known_signals);
      @known_signals = 
         $] >= 5.004 
            ? grep (exists $SIG{$_}, keys %signals)
            : qw(HUP INT QUIT PIPE TERM);

      foreach $sig (@known_signals)
      {
         $SIG{$sig} = \&catch_signal;
      }
   }

}  # &startup

# end of MNI::Startup::startup
1;
